#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures
from oslo_log import log as logging

from tempest.lib.services.volume import base_client

LOG = logging.getLogger(__name__)


class APIMicroversionFixture(fixtures.Fixture):

    def __init__(self, volume_microversion):
        self.volume_microversion = volume_microversion
        new_fixture = (
            'tempest.lib.common.api_microversion_fixture.'
            'APIMicroversionFixture')
        LOG.warning("%s class is deprecated and moved to %s. It"
                    " will be removed in Z cycle.",
                    self.__class__.__name__, new_fixture)

    def _setUp(self):
        super(APIMicroversionFixture, self)._setUp()
        base_client.VOLUME_MICROVERSION = self.volume_microversion
        self.addCleanup(self._reset_volume_microversion)

    def _reset_volume_microversion(self):
        base_client.VOLUME_MICROVERSION = None



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class FloatingIPsClient(base.BaseNetworkClient):

    def create_floatingip(self, **kwargs):
        """Creates a floating IP.

        If you specify port information, associates the floating IP with an
        internal port.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-floating-ip
        """
        uri = '/floatingips'
        post_data = {'floatingip': kwargs}
        return self.create_resource(uri, post_data)

    def update_floatingip(self, floatingip_id, **kwargs):
        """Updates a floating IP and its association with an internal port.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-floating-ip
        """
        uri = '/floatingips/%s' % floatingip_id
        post_data = {'floatingip': kwargs}
        return self.update_resource(uri, post_data)

    def show_floatingip(self, floatingip_id, **fields):
        """Shows details for a floating IP.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-floating-ip-details
        """
        uri = '/floatingips/%s' % floatingip_id
        return self.show_resource(uri, **fields)

    def delete_floatingip(self, floatingip_id):
        uri = '/floatingips/%s' % floatingip_id
        return self.delete_resource(uri)

    def list_floatingips(self, **filters):
        """Lists floating IPs.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-floating-ips
        """
        uri = '/floatingips'
        return self.list_resources(uri, **filters)



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest.lib.services.image.v2.image_cache_client import \
    ImageCacheClient
from tempest.lib.services.image.v2.image_members_client import \
    ImageMembersClient
from tempest.lib.services.image.v2.images_client import ImagesClient
from tempest.lib.services.image.v2.namespace_objects_client import \
    NamespaceObjectsClient
from tempest.lib.services.image.v2.namespace_properties_client import \
    NamespacePropertiesClient
from tempest.lib.services.image.v2.namespace_tags_client import \
    NamespaceTagsClient
from tempest.lib.services.image.v2.namespaces_client import NamespacesClient
from tempest.lib.services.image.v2.resource_types_client import \
    ResourceTypesClient
from tempest.lib.services.image.v2.schemas_client import SchemasClient
from tempest.lib.services.image.v2.tasks_client import TaskClient
from tempest.lib.services.image.v2.versions_client import VersionsClient


__all__ = ['ImageMembersClient', 'ImagesClient', 'ImageCacheClient',
           'NamespaceObjectsClient', 'NamespacePropertiesClient',
           'NamespaceTagsClient', 'NamespacesClient', 'ResourceTypesClient',
           'SchemasClient', 'TaskClient', 'VersionsClient']



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import manage_volume as schema
from tempest.lib.common import rest_client


class VolumeManageClient(rest_client.RestClient):
    """Volume manage client."""

    def manage_volume(self, **kwargs):
        """Manage existing volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#manage-an-existing-volume
        """
        post_body = json.dumps({'volume': kwargs})
        resp, body = self.post('os-volume-manage', post_body)
        body = json.loads(body)
        self.validate_response(schema.manage_volume, resp, body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
https://docs.openstack.org/api-ref/identity/v3/index.html#get-service-catalog
"""

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class CatalogClient(rest_client.RestClient):
    api_version = "v3"

    def show_catalog(self):
        resp, body = self.get('auth/catalog')
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2024 Openstack Foundation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
from tempest.scenario import manager


CONF = config.CONF
LOG = logging.getLogger(__name__)


class TestInstancesWithCinderVolumes(manager.ScenarioTest):
    """This is cinder volumes test.

    Tests are below:
    * test_instances_with_cinder_volumes_on_all_compute_nodes
    """

    compute_min_microversion = '2.60'

    @decorators.idempotent_id('d0e3c1a3-4b0a-4b0e-8b0a-4b0e8b0a4b0e')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'volume', 'image', 'network')
    def test_instances_with_cinder_volumes_on_all_compute_nodes(self):
        """Test instances with cinder volumes launches on all compute nodes

        Steps:
            1. Create an image
            2. Create a keypair
            3. Create a bootable volume from the image and of the given volume
               type
            4. Boot an instance from the bootable volume on each available
               compute node, up to CONF.compute.min_compute_nodes
            5. Create a volume using each volume_types_for_data_volume on all
               available compute nodes, up to CONF.compute.min_compute_nodes.
               Total number of volumes is equal to
               compute nodes * len(volume_types_for_data_volume)
            6. Attach volumes to the instances
            7. Assign floating IP to all instances
            8. Configure security group for ssh access to all instances
            9. Confirm ssh access to all instances
            10. Run write test to all volumes through ssh connection per
                instance
            11. Clean up the sources, an instance, volumes, keypair and image
        """
        boot_volume_type = (CONF.volume.volume_type or
                            self.create_volume_type()['name'])

        # create an image
        image = self.image_create()

        # create keypair
        keypair = self.create_keypair()

        # check all available zones for booting instances
        available_zone = \
            self.os_admin.availability_zone_client.list_availability_zones(
                detail=True)['availabilityZoneInfo']

        hosts = []
        for zone in available_zone:
            if zone['zoneState']['available']:
                for host in zone['hosts']:
                    if 'nova-compute' in zone['hosts'][host] and \
                        zone['hosts'][host]['nova-compute']['available'] and \
                        not host.endswith('-ironic'):
                        hosts.append({'zone': zone['zoneName'],
                                      'host_name': host})

        # fail if there is less hosts than minimal number of instances
        if len(hosts) < CONF.compute.min_compute_nodes:
            raise exceptions.InvalidConfiguration(
                "Host list %s is shorter than min_compute_nodes. " % hosts)

        # get volume types
        volume_types = []
        if CONF.volume_feature_enabled.volume_types_for_data_volume:
            types = CONF.volume_feature_enabled.volume_types_for_data_volume
            volume_types = types.split(',')
        else:
            # no user specified volume types, create 2 default ones
            volume_types.append(self.create_volume_type()['name'])
            volume_types.append(self.create_volume_type()['name'])

        hosts_to_boot_servers = hosts[:CONF.compute.min_compute_nodes]
        LOG.debug("List of hosts selected to boot servers %s: ",
                  hosts_to_boot_servers)

        # create volumes so that we dont need to wait for them to be created
        # and save them in a list
        created_volumes = []
        for host in hosts_to_boot_servers:
            for volume_type in volume_types:
                created_volumes.append(
                    self.create_volume(volume_type=volume_type,
                                       wait_until=None)
                )

        bootable_volumes = []
        for host in hosts_to_boot_servers:
            # create boot volume from image and of the given volume type
            bootable_volumes.append(
                self.create_volume(
                    imageRef=image, volume_type=boot_volume_type,
                    wait_until=None)
            )

        # boot server
        servers = []

        for bootable_volume in bootable_volumes:

            # wait for bootable volumes to become available
            waiters.wait_for_volume_resource_status(
                self.volumes_client, bootable_volume['id'], 'available')

            # create an instance from bootable volume
            server = self.boot_instance_from_resource(
                source_id=bootable_volume['id'],
                source_type='volume',
                keypair=keypair,
                wait_until=None
            )
            servers.append(server)

        start = 0
        end = len(volume_types)
        for server in servers:
            attached_volumes = []

            # wait for server to become active
            waiters.wait_for_server_status(self.servers_client,
                                           server['id'], 'ACTIVE')

            # attach volumes to the instances
            for volume in created_volumes[start:end]:

                # wait for volume to become available
                waiters.wait_for_volume_resource_status(
                    self.volumes_client, volume['id'], 'available')

                attached_volume = self.nova_volume_attach(server, volume)
                attached_volumes.append(attached_volume)
                LOG.debug("Attached volume %s to server %s",
                          attached_volume['id'], server['id'])

            # assign floating ip
            floating_ip = None
            if (CONF.network_feature_enabled.floating_ips and
                CONF.network.floating_network_name):
                fip = self.create_floating_ip(server)
                floating_ip = self.associate_floating_ip(
                    fip, server)
                ssh_ip = floating_ip['floating_ip_address']
            else:
                ssh_ip = self.get_server_ip(server)

            # create security group
            self.create_and_add_security_group_to_server(server)

            # confirm ssh access
            self.linux_client = self.get_remote_client(
                ssh_ip, private_key=keypair['private_key'],
                server=server
            )

            # run write test on all volumes
            for volume in attached_volumes:

                waiters.wait_for_volume_resource_status(
                    self.volumes_client, volume['id'], 'in-use')

                # get the mount path
                mount_path = f"/mnt/{volume['attachments'][0]['device'][5:]}"

                # create file for mounting on server
                self.create_file(ssh_ip, mount_path,
                                 private_key=keypair['private_key'],
                                 server=server)

                # dev name volume['attachments'][0]['device'][5:] is like
                # /dev/vdb, we need to remove /dev/ -> first 5 chars
                timestamp_before = self.create_timestamp(
                    ssh_ip, private_key=keypair['private_key'], server=server,
                    dev_name=volume['attachments'][0]['device'][5:],
                    mount_path=mount_path
                )
                timestamp_after = self.get_timestamp(
                    ssh_ip, private_key=keypair['private_key'], server=server,
                    dev_name=volume['attachments'][0]['device'][5:],
                    mount_path=mount_path
                )
                self.assertEqual(timestamp_before, timestamp_after)

                # delete volume
                self.nova_volume_detach(server, volume)
                self.volumes_client.delete_volume(volume['id'])

            if floating_ip:
                # delete the floating IP, this should refresh the server
                # addresses
                self.disassociate_floating_ip(floating_ip)
                waiters.wait_for_server_floating_ip(
                    self.servers_client, server, floating_ip,
                    wait_for_disassociate=True)

            start += len(volume_types)
            end += len(volume_types)



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest.lib.services.identity.v2.endpoints_client import EndpointsClient
from tempest.lib.services.identity.v2.identity_client import IdentityClient
from tempest.lib.services.identity.v2.roles_client import RolesClient
from tempest.lib.services.identity.v2.services_client import ServicesClient
from tempest.lib.services.identity.v2.tenants_client import TenantsClient
from tempest.lib.services.identity.v2.token_client import TokenClient
from tempest.lib.services.identity.v2.users_client import UsersClient

__all__ = ['EndpointsClient', 'IdentityClient', 'RolesClient',
           'ServicesClient', 'TenantsClient', 'TokenClient', 'UsersClient']



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import re

RE_OWNER = re.compile('^network:.*router_.*interface.*')


def _is_owner_router_interface(owner):
    return bool(RE_OWNER.match(owner))


def is_router_interface_port(port):
    """Based on the port attributes determines is it a router interface."""
    return _is_owner_router_interface(port['device_owner'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_75 import servers as servers275


###########################################################################
#
# 2.79:
#
# The delete_on_termination parameter is now returned in the response body
# of the following calls:
#
# - GET /servers/{server_id}/os-volume_attachments
# - GET /servers/{server_id}/os-volume_attachments/{volume_id}
# - POST /servers/{server_id}/os-volume_attachments
###########################################################################

attach_volume = copy.deepcopy(servers275.attach_volume)
attach_volume['response_body']['properties']['volumeAttachment'][
    'properties'].update({'delete_on_termination': {'type': 'boolean'}})
attach_volume['response_body']['properties']['volumeAttachment'][
    'required'].append('delete_on_termination')

show_volume_attachment = copy.deepcopy(servers275.show_volume_attachment)
show_volume_attachment['response_body']['properties']['volumeAttachment'][
    'properties'].update({'delete_on_termination': {'type': 'boolean'}})
show_volume_attachment['response_body']['properties'][
    'volumeAttachment']['required'].append('delete_on_termination')

list_volume_attachments = copy.deepcopy(servers275.list_volume_attachments)
list_volume_attachments['response_body']['properties']['volumeAttachments'][
    'items']['properties'].update(
        {'delete_on_termination': {'type': 'boolean'}})
list_volume_attachments['response_body']['properties'][
    'volumeAttachments']['items']['required'].append('delete_on_termination')

# NOTE(zhufl): Below are the unchanged schema in this microversion. We
# need to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.75 ***
rebuild_server = copy.deepcopy(servers275.rebuild_server)
rebuild_server_with_admin_pass = copy.deepcopy(
    servers275.rebuild_server_with_admin_pass)
update_server = copy.deepcopy(servers275.update_server)
get_server = copy.deepcopy(servers275.get_server)
list_servers_detail = copy.deepcopy(servers275.list_servers_detail)
list_servers = copy.deepcopy(servers275.list_servers)
show_server_diagnostics = copy.deepcopy(servers275.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers275.get_remote_consoles)
list_tags = copy.deepcopy(servers275.list_tags)
update_all_tags = copy.deepcopy(servers275.update_all_tags)
delete_all_tags = copy.deepcopy(servers275.delete_all_tags)
check_tag_existence = copy.deepcopy(servers275.check_tag_existence)
update_tag = copy.deepcopy(servers275.update_tag)
delete_tag = copy.deepcopy(servers275.delete_tag)
show_instance_action = copy.deepcopy(servers275.show_instance_action)
create_backup = copy.deepcopy(servers275.create_backup)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_57 import servers as servers257
from tempest.lib.api_schema.response.compute.v2_73 import servers as servers273


###########################################################################
#
# 2.75:
#
# Server representation is made consistent among GET, PUT
# and Rebuild serevr APIs response.
#
###########################################################################

rebuild_server = copy.deepcopy(servers273.get_server)
rebuild_server['response_body']['properties']['server'][
    'properties'].pop('OS-EXT-SRV-ATTR:user_data')
rebuild_server['status_code'] = [202]
rebuild_server['response_body']['properties']['server'][
    'properties'].update({'user_data': servers257.user_data})
rebuild_server['response_body']['properties']['server'][
    'required'].append('user_data')

rebuild_server_with_admin_pass = copy.deepcopy(rebuild_server)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'adminPass': {'type': 'string'}})
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'required'].append('adminPass')

update_server = copy.deepcopy(servers273.get_server)

# NOTE(gmann): Below are the unchanged schema in this microversion. We
# need to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.73 ***
get_server = copy.deepcopy(servers273.get_server)
list_servers = copy.deepcopy(servers273.list_servers)
list_servers_detail = copy.deepcopy(servers273.list_servers_detail)
show_server_diagnostics = copy.deepcopy(servers273.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers273.get_remote_consoles)
list_tags = copy.deepcopy(servers273.list_tags)
update_all_tags = copy.deepcopy(servers273.update_all_tags)
delete_all_tags = copy.deepcopy(servers273.delete_all_tags)
check_tag_existence = copy.deepcopy(servers273.check_tag_existence)
update_tag = copy.deepcopy(servers273.update_tag)
delete_tag = copy.deepcopy(servers273.delete_tag)
attach_volume = copy.deepcopy(servers273.attach_volume)
show_volume_attachment = copy.deepcopy(servers273.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers273.list_volume_attachments)
show_instance_action = copy.deepcopy(servers273.show_instance_action)
create_backup = copy.deepcopy(servers273.create_backup)



# Copyright 2022 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import signal
import subprocess

import fixtures

from oslo_log import log


LOG = log.getLogger(__name__)


class NetDowntimeMeter(fixtures.Fixture):
    def __init__(self, dest_ip, interval='0.2'):
        self.dest_ip = dest_ip
        # Note: for intervals lower than 0.2 ping requires root privileges
        self.interval = interval
        self.ping_process = None

    def _setUp(self):
        self.start_background_pinger()

    def start_background_pinger(self):
        cmd = ['ping', '-q', '-s1']
        cmd.append('-i{}'.format(self.interval))
        cmd.append(self.dest_ip)
        LOG.debug("Starting background pinger to '{}' with interval {}".format(
            self.dest_ip, self.interval))
        self.ping_process = subprocess.Popen(
            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        self.addCleanup(self.cleanup)

    def cleanup(self):
        if self.ping_process and self.ping_process.poll() is None:
            LOG.debug('Terminating background pinger with pid {}'.format(
                self.ping_process.pid))
            self.ping_process.terminate()
        self.ping_process = None

    def get_downtime(self):
        self.ping_process.send_signal(signal.SIGQUIT)
        # Example of the expected output:
        # 264/274 packets, 3% loss
        output = self.ping_process.stderr.readline().strip().decode('utf-8')
        if output and len(output.split()[0].split('/')) == 2:
            succ, total = output.split()[0].split('/')
            return (int(total) - int(succ)) * float(self.interval)
        else:
            LOG.warning('Unexpected output obtained from the pinger: %s',
                        output)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc


class NamespacesClient(rest_client.RestClient):
    api_version = "v2"

    def create_namespace(self, **kwargs):
        """Create a namespace.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#create-namespace
        """
        data = json.dumps(kwargs)
        resp, body = self.post('metadefs/namespaces', data)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_namespaces(self):
        """List namespaces

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#list-namespaces
        """
        url = 'metadefs/namespaces'
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_namespace(self, namespace):
        """Show namespace details.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#get-namespace-details
        """
        url = 'metadefs/namespaces/%s' % namespace
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_namespace(id)
        except lib_exc.NotFound:
            return True
        return False

    def update_namespace(self, namespace, **kwargs):
        """Update a namespace.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#update-namespace
        """
        # NOTE: On Glance API, we need to pass namespace on both URI
        # and a request body.
        params = {'namespace': namespace}
        params.update(kwargs)
        data = json.dumps(params)
        url = 'metadefs/namespaces/%s' % namespace
        resp, body = self.put(url, body=data)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_namespace(self, namespace):
        """Delete a namespace.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#delete-namespace
        """
        url = 'metadefs/namespaces/%s' % namespace
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)
